﻿'--------------------------------------------------------------------------------
'--        File: ExifToolDemo.vb
'--
'-- Description: Demo program to show usage of the ExifToolIO VB.Net library 
'--              interface to Perl exiftool application script by Phil Harvey.
'--              See http://owl.phy.queensu.ca/~phil/exiftool/ for details.
'--              This is just a simple program to show how to use the ExifToolIO.dll library.
'--              It is not feature rich or optimal and does not have much error checking.
'--
'--     License: This is software, in whole or part, is free for use in
'--              non-commercial applications, provided that this copyright notice
'--              is retained.  A licensing fee may be required for use in a
'--              commercial application.
'--
'--     History: 
'-- 2014-06-09 - NCC - Created,  curtis1757 at gmail dot com
'--------------------------------------------------------------------------------

Imports ExifToolLib
Imports System.IO

Public Class ExifToolDemo
    Private LastTitle As String = ""
    Private LastcolValueWidth As Integer
    Private Loaded As Boolean = False
    Private FileTagList As New List(Of FileTagValues)

    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        LastcolValueWidth = Me.colValue.Width

        ' Add any initialization after the InitializeComponent() call.

        StartExifTool()
    End Sub

    Private Sub ExifToolDemo_Load(sender As Object, e As EventArgs) Handles Me.Load
        Loaded = True
    End Sub

    Private Sub StartExifTool()
        If ExifToolIO.Initiailize() Then
            Me.Text = "ExifToolIO.dll demo and test program - exiftool version currently loaded: " & ExifToolIO.LoadedVersion
            Me.cbxUseResourcesExifToolExe.Text = "Use exiftool.exe from resources? (version " & ExifToolIO.ResourcesVersion & ")"
        Else
            MsgBox("Exiftool initialization failed")
            Application.Exit()
        End If
    End Sub

    Private Sub ExifToolDemo_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing

        Dim ExifToolReturnCode As Integer = ExifToolIO.Close()

        '--Normally Exiftool closes fine... but if not the exit code exiftool returns can be checked 

    End Sub

    '--Pick files to use

    Private Sub btnGetFiles_Click(sender As Object, e As EventArgs) Handles btnGetFiles.Click
        Dim OpenDialog As New OpenFileDialog With {.Title = "Select file(s) to get tags from",
                                                   .Filter = "jpg files (*.jpg)|*.jpg|All files (*.*)|*.*",
                                                   .Multiselect = True}

        If OpenDialog.ShowDialog() = DialogResult.OK AndAlso OpenDialog.FileNames.Count > 0 Then
            Me.lbxFiles.Items.Clear()
            Me.lbxFiles.Items.AddRange(OpenDialog.FileNames)
        End If
    End Sub

    '--Get tags for each of the picked files. Hpow tags are gotten is determined by how many tags are requested by in TagList

    Private Sub GetTagValues(TagList() As String, Title As String)
        If Me.lbxFiles.Items.Count > 0 Then
            FileTagList.Clear()

            LastTitle = Title

            Me.lblInfo.Text = ""
            Me.tbxErrorMsgs.Text = ""

            Dim starttime As Long = Now.Ticks
            Dim ntag As Integer = 0

            For Each fn As String In Me.lbxFiles.Items
                Dim tvalues As New FileTagValues(fn, TagList)
                Me.tbxErrorMsgs.Text &= ExifToolIO.LastErrorMsg
                FileTagList.Add(tvalues)

                ntag += tvalues.TagValueList.Count

                Me.lblInfo.Text &= "."
                If Me.lblInfo.Text.Length > 100 Then Me.lblInfo.Text = ""
            Next

            Dim msec As Double = CDbl(Now.Ticks - starttime) / CDbl(TimeSpan.TicksPerMillisecond)

            Me.lblInfo.Text = "Retrieved " & Title & " in " & Me.lbxFiles.Items.Count.ToString("#,###") &
                              " files with " & ntag.ToString("#,###") &
                              " tags, in " & msec.ToString("#,###.0") & " ms, = " &
                              (msec / Me.lbxFiles.Items.Count).ToString("#,###.0") & " ms per file"

            If Me.lbxFiles.SelectedIndex < 0 Then Me.lbxFiles.SelectedIndex = 0

            ShowTagValues()
        End If
    End Sub

    Private Sub btnGetAllTags_Click(sender As Object, e As EventArgs) Handles btnGetAllTags.Click

        '--Get all tags

        GetTagValues(Nothing, "Value and PrintValue for ALL tags")
        Me.colValue.Width = LastcolValueWidth
    End Sub

    Private Sub btnGetSomeTags_Click(sender As Object, e As EventArgs) Handles btnGetSomeTags.Click

        '--Here are a few tags to get, just a simple hardcoded example

        GetTagValues({"IFD0:Artist", "IFD0:XPAuthor", "System:FileName", "System:FileSize", "XMP-mwg-rs:RegionName"}, "PrintValue for some tags")

        If Me.colValue.Width > 0 Then
            LastcolValueWidth = Me.colValue.Width
            Me.colValue.Width = 0
        End If
    End Sub

    Private Sub btnGetOneTag_Click(sender As Object, e As EventArgs) Handles btnGetOneTag.Click

        '--Here just get one tag

        GetTagValues({"IFD0:Artist"}, "PrintValue for one tag")

        If Me.colValue.Width > 0 Then
            LastcolValueWidth = Me.colValue.Width
            Me.colValue.Width = 0
        End If
    End Sub

    Private Sub lbxFiles_SelectedIndexChanged(sender As Object, e As EventArgs) Handles lbxFiles.SelectedIndexChanged
        ShowTagValues()
    End Sub

    '--Load of the ListView with our tag values, Group them by the Tag Group name
    '--For TagValues that are arrays, append the array indice to the tag name

    Private Sub ShowTagValues()
        If FileTagList.Count = 0 Then Return

        Me.lvTagList.Items.Clear()
        Dim values As List(Of TagValue) = FileTagList(Me.lbxFiles.SelectedIndex).TagValueList

        Me.gbxTagValues.Text = LastTitle & " for file:  " & Path.GetFileName(FileTagList(Me.lbxFiles.SelectedIndex).FileName)

        For Each tv As TagValue In values
            Dim group As ListViewGroup = Nothing
            For Each g As ListViewGroup In Me.lvTagList.Groups
                If g.Header = tv.Group Then
                    group = g
                    Exit For
                End If
            Next

            If group Is Nothing Then
                group = New ListViewGroup(tv.Group)
                Me.lvTagList.Groups.Add(group)
            End If

            If tv.PrintValue.Count > 1 Then
                For i As Integer = 0 To tv.PrintValue.Count - 1
                    If tv.Value Is Nothing Then
                        Me.lvTagList.Items.Add(New ListViewItem({tv.Name & "(" & i & ")", "", tv.PrintValue(i)}, group))
                    Else
                        Me.lvTagList.Items.Add(New ListViewItem({tv.Name & "(" & i & ")", tv.Value(i), tv.PrintValue(i)}, group))
                    End If
                Next
            Else
                If tv.Value Is Nothing Then
                    Me.lvTagList.Items.Add(New ListViewItem({tv.Name, "", tv.PrintValue(0)}, group))
                Else
                    Me.lvTagList.Items.Add(New ListViewItem({tv.Name, tv.Value(0), tv.PrintValue(0)}, group))
                End If
            End If
        Next
    End Sub

    Private Sub cbxDeleteTempProcess_CheckedChanged(sender As Object, e As EventArgs) Handles cbxDeleteTempProcess.CheckedChanged
        If Not Loaded Then Return

        ExifToolIO.Options.DeleteTempProcesses = Me.cbxDeleteTempProcess.Checked
        ExifToolIO.Close()
        StartExifTool()
    End Sub

    Private Sub cbxUseStdInp_CheckedChanged(sender As Object, e As EventArgs) Handles cbxUseStdInp.CheckedChanged
        If Not Loaded Then Return

        If Me.cbxUseStdInp.Checked Then
            ExifToolIO.Close()
            ExifToolIO.Options.UseStdInpCommandFile = True
            StartExifTool()
        Else
            Dim SaveDialog As New SaveFileDialog With {.Title = "Select file name to use for ExifTool Command Input"}
            If SaveDialog.ShowDialog() = DialogResult.OK Then
                ExifToolIO.Close()

                ExifToolIO.Options.UseStdInpCommandFile = False
                ExifToolIO.Options.CommandFileName = SaveDialog.FileName
                Me.tbxCmdInpFile.Text = SaveDialog.FileName

                StartExifTool()
            End If
        End If
    End Sub

    Private Sub cbxUseResourcesExifToolExe_CheckedChanged(sender As Object, e As EventArgs) Handles cbxUseResourcesExifToolExe.CheckedChanged
        If Not Loaded Then Return

        If Me.cbxUseResourcesExifToolExe.Checked Then
            ExifToolIO.Close()
            ExifToolIO.Options.UseResourcesExifToolExe = True
            StartExifTool()
        Else
            Dim OpenDialog As New OpenFileDialog With {.Title = "Select exiftool.exe to use", .Filter = "exe files (*.exe)|*.exe", .CheckFileExists = True}
            If OpenDialog.ShowDialog() = DialogResult.OK Then
                ExifToolIO.Close()

                ExifToolIO.Options.UseResourcesExifToolExe = False
                ExifToolIO.Options.ExifToolExeFileName = OpenDialog.FileName
                Me.tbxExifToolExeFn.Text = OpenDialog.FileName

                StartExifTool()
            End If
        End If
    End Sub

    Private Sub btnDisplayCmdFile_Click(sender As Object, e As EventArgs) Handles btnDisplayCmdFile.Click
        If File.Exists(Me.tbxCmdInpFile.Text) Then
            Dim cmdfileform As New ShowText

            Using fs As New FileStream(Me.tbxCmdInpFile.Text, FileMode.Open, FileAccess.Read, FileShare.ReadWrite)
                Using sr As New StreamReader(fs)
                    cmdfileform.tbxText.Text = sr.ReadToEnd
                End Using
            End Using

            cmdfileform.tbxText.Focus()
            cmdfileform.tbxText.Select(cmdfileform.tbxText.Text.Length, 0)
            cmdfileform.tbxText.ScrollToCaret()
            cmdfileform.Text = "exiftool command file: " & Me.tbxCmdInpFile.Text

            cmdfileform.ShowDialog()
        End If
    End Sub

    Private Sub btnDisplayXML_Click(sender As Object, e As EventArgs) Handles btnDisplayXML.Click

        If FileTagList.Count > 0 AndAlso FileTagList(Me.lbxFiles.SelectedIndex).AllTagsXML <> "" Then
            Dim XMLTextForm As New ShowText

            XMLTextForm.tbxText.Text = FileTagList(Me.lbxFiles.SelectedIndex).AllTagsXML

            XMLTextForm.tbxText.Focus()
            XMLTextForm.tbxText.Select(XMLTextForm.tbxText.Text.Length, 0)
            XMLTextForm.tbxText.ScrollToCaret()
            XMLTextForm.Text = "XML"

            XMLTextForm.ShowDialog()
        End If
    End Sub

    Private Sub btnJSONOutput_Click(sender As Object, e As EventArgs) Handles btnJSONOutput.Click

        If FileTagList.Count > 0 Then
            Dim fn As String = FileTagList(Me.lbxFiles.SelectedIndex).FileName
            Dim JSONTextForm As New ShowText

            ExifToolIO.Cmd("-L -j -all")
            JSONTextForm.tbxText.Text = ExifToolIO.Execute(fn)

            JSONTextForm.tbxText.Focus()
            JSONTextForm.tbxText.Select(JSONTextForm.tbxText.Text.Length, 0)
            JSONTextForm.tbxText.ScrollToCaret()
            JSONTextForm.Text = "JSON for " & Path.GetFileName(fn)

            JSONTextForm.ShowDialog()
        End If
    End Sub
End Class

'--Simple Class to hold the value for a single tag (could be an array value)

Public Class TagValue
    Public Group As String
    Public Name As String
    Public Value() As String
    Public PrintValue() As String
End Class

'--Simple Class to hold all of the tags for a single file.
'--Three differnt methods of getting tag values are used here to illustrate different ways to get the values.
'--the GetAllTags method uses the -X option to get XML output from exiftool which has the nost infomation as 
'--compared to other output formats that Exiftool can generate

Public Class FileTagValues
    Public FileName As String
    Public TagValueList As List(Of TagValue)

    Public Sub New(fn As String, TagsToGet() As String)
        FileName = fn

        '--Get tag value(s) by different methods (based on number of tags to get)
        '--Just to demonstrate different ways to get tag values

        If TagsToGet Is Nothing OrElse TagsToGet.Length = 0 Then
            TagValueList = GetAllExistingTags(fn)
        ElseIf TagsToGet.Length > 1 Then
            TagValueList = GetSomeTags(fn, TagsToGet)
        Else
            TagValueList = GetOneTag(fn, TagsToGet(0))
        End If
    End Sub

    Private Const CmdOverWriteOriginal = "-overwrite_original"
    Private Const CmdQuiet = "-q"
    Private Const CmdPrintFormat = "-p"
    Private Const CmdSep = "-sep"                   '--set seperator string for between array values
    Private Const CmdDisablePrintConversion = "-n"  '--use this command to get 'real' value
    Private Const CmdXMLOutput = "-X -t -l"         '--XML output with table names and get prt and val values at same time
    Private Const TagValSep = "<!>"

    '--Example of how to get one tag (just PrintValue) (which could be an array of values for some tags like XMP-mwg-rs:RegionName)

    Public Function GetOneTag(fn As String, TagName As String) As List(Of TagValue)

        Dim NameValues As New List(Of TagValue)

        ExifToolIO.Cmd(CmdQuiet)
        ExifToolIO.Cmd(CmdSep)
        ExifToolIO.Cmd(TagValSep)
        ExifToolIO.Cmd(CmdPrintFormat)
        ExifToolIO.Cmd("$" & TagName, False)
        Dim printvalresult As String = ExifToolIO.Execute(fn).Trim

        NameValues.Add(New TagValue With {.Group = TagName.Split(":")(0),
                                        .Name = TagName.Split(":")(1),
                                        .Value = Nothing,
                                        .PrintValue = Split(printvalresult.Trim, TagValSep)})
  
        Return NameValues
    End Function

    '--Example of how to get several tags at once (just PrintValue)

    Public Function GetSomeTags(fn As String, TagNameList() As String, Optional GetOnlyPrintConversionValues As Boolean = True) As List(Of TagValue)
        Const ValueSepStr As String = "<!alSKdjFHg!>"   '--just a very unique string that should not be in any tag values

        ExifToolIO.Cmd(CmdQuiet)
        ExifToolIO.Cmd(CmdSep)
        ExifToolIO.Cmd(TagValSep)

        '--Now make -PrintFormat cmds with the unique string ValueSepStr on first line, then $<TagName> on next line,
        '--This way we can later split the output based on ValueSepStr and Tags that don't exist or are empty will be = to string.empty

        For Each tn As String In TagNameList
            ExifToolIO.Cmd(CmdPrintFormat)
            ExifToolIO.Cmd(ValueSepStr, False)
            ExifToolIO.Cmd(CmdPrintFormat)
            ExifToolIO.Cmd("$" & tn, False)
        Next

        '--Get tag values with Exiftool and split the output using ValueSepStr as delimiter and remove all CR's and LF's

        Dim printvalresults() As String = Split(ExifToolIO.Execute(fn).Substring(ValueSepStr.Length).Replace(ControlChars.Lf, String.Empty).Replace(ControlChars.Cr, String.Empty), ValueSepStr)

        Dim NameValues As New List(Of TagValue)
        For i As Integer = 0 To printvalresults.GetUpperBound(0)
            NameValues.Add(New TagValue With {.Group = TagNameList(i).Split(":")(0),
                                              .Name = TagNameList(i).Split(":")(1),
                                              .Value = Nothing,
                                              .PrintValue = Split(printvalresults(i).Trim, TagValSep)})
        Next

        Return NameValues
    End Function

    Private XML As String

    Public ReadOnly Property AllTagsXML As String
        Get
            Return XML
        End Get
    End Property

    '--XML names used by exiftool -X output

    Private ReadOnly rdfNS As XNamespace = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    Private ReadOnly rdfRDF As XName = rdfNS + "RDF"
    Private ReadOnly rdfDescription As XName = rdfNS + "Description"
    Private ReadOnly rdfBag As XName = rdfNS + "Bag"
    Private ReadOnly rdfli As XName = rdfNS + "li"

    Private ReadOnly etNS As XNamespace = "http://ns.exiftool.ca/1.0/"
    Private ReadOnly etprt As XName = etNS + "prt"
    Private ReadOnly etval As XName = etNS + "val"

    '--Example of how to get all tags at once using the XML output format from exiftool (both Printvalue and Value)

    Private Function GetAllExistingTags(fn As String) As List(Of TagValue)
        Dim NameValues As New List(Of TagValue)

        If Not File.Exists(fn) Then Return NameValues

        ExifToolIO.Cmd(CmdQuiet)

        '--Note: spaces in CmdXMLOutput are converted to CrLF so each command is on a seprate line as required by exiftool

        ExifToolIO.Cmd(CmdXMLOutput)

        XML = ExifToolIO.Execute(fn)
        Dim TagsXML As XElement = XDocument.Parse(XML).Element(rdfRDF).Element(rdfDescription)

        '--create NameSpace table to be able to translate prefixes

        Dim nstable As New Dictionary(Of XNamespace, String)
        For Each a As XAttribute In TagsXML.Attributes
            nstable.Add(a.Value, a.Name.LocalName)
        Next

        '--Loop through all the tag Elements

        For Each tt As XElement In TagsXML.Elements
            Dim t As XElement = tt.Element(rdfDescription)

            '--We should always get a et:prt value

            Dim pv As List(Of String)
            If t.Element(etprt).Element(rdfBag) Is Nothing Then
                pv = {t.Element(etprt).Value}.ToList
            Else

                '--We have an RDF:Bag so get array elements

                pv = New List(Of String)
                For Each xe As XElement In t.Element(etprt).Element(rdfBag).Elements(rdfli)
                    pv.Add(xe.Value)
                Next
            End If

            '--Now check to see if we have a et:val element, if not then the et:prt value is the same and use it

            Dim nv As List(Of String)
            If t.Element(etval) Is Nothing Then
                nv = New List(Of String)
                For Each s As String In pv
                    nv.Add(s)
                Next
            Else

                '--We have a et:val element, get it's value or the rdf:Bag of values

                If t.Element(etval).Element(rdfBag) Is Nothing Then
                    nv = {t.Element(etval).Value}.ToList
                Else

                    '--We have an RDF:Bag so get array elements

                    nv = New List(Of String)
                    For Each xe As XElement In t.Element(etval).Element(rdfBag).Elements(rdfli)
                        nv.Add(xe.Value)
                    Next
                End If
            End If

            '--Add tag value to our list

            NameValues.Add(New TagValue With {.Group = nstable(tt.Name.NamespaceName),
                                              .Name = tt.Name.LocalName,
                                              .Value = nv.ToArray,
                                              .PrintValue = pv.ToArray})
        Next

        Return NameValues
    End Function
End Class


